#include "MagData.h"
#include "Consts.h"

MagData::MagData()
    :
    m_hasData(false),
    m_hasGeomData(false),
    m_mag(0),
    m_phase(0),
    m_range(0)
{
}

void MagData::SetOpticData(double stdphase, double stdtrange, double stdmag)
{
    m_hasData = true;
    m_mag = 0;
    m_phase = 0;
    m_range = 0;
    m_stdphase = stdphase;
    m_stdrange = stdtrange;
    m_stdmag = stdmag;
}

void MagData::SetGeomData(double rcs, double albedo)
{
    m_hasGeomData = true;
    m_mag = 0;
    m_rcs = rcs;
    m_albedo = albedo;
}

bool MagData::Load(myXML::Contain* parent)
{
    LoadXML(parent, "Phase", m_stdphase);
    LoadXML(parent, "Range", m_stdrange);
    LoadXML(parent, "Stdmag", m_stdmag, false, &m_hasData);

    return true;
}

void MagData::load(const jsonval& parent)
{
    loadjson(parent, "phase", m_stdphase);
    loadjson(parent, "range", m_stdrange);
    loadjson(parent, "mag", m_stdmag);
    m_hasData = true;
}

double MagData::CalcA0g(double stdmag, double stdphase, double stdrange, double sunMag)
{
    double C = pow(10.0, ((sunMag - stdmag) / 2.5));
    double F = 2 / 3.0 * sqr(Pi) * ((Pi - stdphase) * cos(stdphase) + sin(stdphase)) * (1 - stdphase / Pi);
    return C * sqr(stdrange * 1E6) / F;
}

double MagData::CalcRealMag(double A0g, double sunMag, double phase, double range)
{
    double F = 2 / 3.0 * sqr(Pi) * ((Pi - phase) * cos(phase) + sin(phase)) * (1 - phase / Pi);
    return sunMag - 2.5 * log10(F * A0g / sqr(range * 1E6));
}

void MagData::CalcMag(double phase, double range)
{
    double sunMag = -26.58;
    m_mag = CalcRealMag(CalcA0g(m_stdmag, m_stdphase, m_stdrange, sunMag), sunMag, phase, range);
    m_phase = phase;
    m_range = range;
}

void MagData::CalcGeomMag(double phase, double range)
{
    double sunMag = -26.58;
    double F = 2 / 3.0 * sqr(Pi) * ((Pi - phase) * cos(phase) + sin(phase)) * (1 - phase / Pi);
    double A0g = m_rcs * 0.15;
    m_mag = sunMag - 2.5 * log10(F * A0g / sqr(range * 1E6));
    m_phase = phase;
    m_range = range;
}
