#ifndef PROPINFOBASE_H
#define PROPINFOBASE_H

#include <myXML_Document.hpp>
#include "DVectors.h"

#include <map>
#include "AtmModel.h"
#include "EarthModel.h"
#include "json_functions.h"

enum proptypes
{
    ST_PROP = 0,
    KEPLER_PROP,
    PROGNOZ_PROP,
    SGP_PROP
};

class PropInfoBase
{
public:
    PropInfoBase()
    {
    };
    PropInfoBase(std::string lightModel, const EarthModel& earthModel, const AtmModel& atmModel, bool sun, bool moon, int type);

    bool Load(myXML::Contain* parent);
    void load(const jsonval& parent);
    bool IsSun() const { return m_sun; }
    bool IsMoon() const { return m_moon; }
    int GetType() const { return m_type; }
    const AtmModel& GetAtmModel() const { return m_atmModel; }
    const EarthModel& GetEarthModel() const { return m_earthModel; }
    std::string GetLightModel() const { return m_lightModel; }
    std::string GetAsXML() const;
    void ToXml(std::ostream& outs) const;
private:

    static std::string TypeConvert(int value);
    static int TypeConvert(const std::string& value);

    AtmModel m_atmModel;
    EarthModel m_earthModel;
    std::string m_lightModel;
    bool m_sun;
    bool m_moon;
    int m_type;
    int m_id;
};

typedef std::map<int, PropInfoBase> PropInfoBases;
typedef PropInfoBase MotionModel;

#endif
