#include "orbBlock.h"
#include <utility>
#include "TextFunctions.h"

OrbBlock::OrbBlock()
    :
    m_ip(nullptr)
{
}

OrbBlock::OrbBlock(
    const SInitOrbit& orbit,
    const TimeJD& beg,
    const TimeJD& end
)
    :
    m_orbit(orbit),
    m_beg(beg),
    m_end(end),
    m_mind(-1),
    m_ip(nullptr)
{
#ifdef _DEBUG
    m_beg_s = DateToStr(beg);
    m_end_s = DateToStr(end);
#endif
}

OrbBlock::OrbBlock(
    const SInitOrbit& orbit,
    const TimeJD& beg,
    const TimeJD& end,
    const double mind,
    std::string next_orbitid
)
    :
    m_orbit(orbit),
    m_beg(beg),
    m_end(end),
    m_mind(mind),
    m_next_orbitid(std::move(next_orbitid)),
    m_ip(nullptr)
{
#ifdef _DEBUG
    m_beg_s = DateToStr(beg);
    m_end_s = DateToStr(end);
#endif
}

OrbBlock::~OrbBlock()
{
    delete m_ip;
}

void OrbBlock::interpolate(const TimeJD& beg, const TimeJD& end)
{
    m_ip = new XInterpolator(m_orbit, beg, end);
}

void OrbBlock::load(const jsonval& parent)
{
    std::string orbitid;
    loadjson(parent, "id", orbitid);
    loadjson(parent, "beg", m_beg);
    loadjson(parent, "end", m_end);
    loadjson(parent, "mind_km", m_mind, false);

    std::string vecs;
    loadjson(parent, "vecs", vecs, false);

    static std::vector<std::string> arr = splitstr(vecs, ",");

    std::vector<IRes> vres;
    for (size_t i = 0; i < arr.size(); i += 8) {
        int days;
        Converter::ToNumber(arr[i], days);

        double fraction;
        Converter::ToNumber(arr[i + 1], fraction);

        double x, y, z, vx, vy, vz;
        Converter::ToNumber(arr[i + 2], x);
        Converter::ToNumber(arr[i + 3], y);
        Converter::ToNumber(arr[i + 4], z);
        Converter::ToNumber(arr[i + 5], vx);
        Converter::ToNumber(arr[i + 6], vy);
        Converter::ToNumber(arr[i + 7], vz);

        vres.emplace_back(Vect6(x, y, z, vx, vy, vz), TimeJD(days, fraction), 0, orbitid);
    }
}
