#pragma once

#include "TM.h"
#include "StateVector.h"
#include "interpolate.h"
#include "json_functions.h"

class OrbBlock {
public:
    OrbBlock();
    OrbBlock(const SInitOrbit& orbit, const TimeJD& beg, const TimeJD& end);
    OrbBlock(const SInitOrbit& orbit, const TimeJD& beg, const TimeJD& end, double mind, std::string next_orbitid);
    ~OrbBlock();

    TimeJD get_beg() const { return m_beg; }
    TimeJD get_end() const { return m_end; }
    const SInitOrbit& get_orbit() const { return m_orbit; }
    double get_min_distance() const { return m_mind; }
    std::string get_next_orbitid() const { return m_next_orbitid; }
    void interpolate(const TimeJD& beg, const TimeJD& end);
    const XInterpolator* get_interpolator() const { return m_ip; }
    void load(const jsonval& parent);
private:
    SInitOrbit m_orbit;
    TimeJD m_beg;
    TimeJD m_end;
    double m_mind;
    std::string m_next_orbitid;
    XInterpolator* m_ip;

#ifdef _DEBUG
    std::string m_beg_s;
    std::string m_end_s;
#endif
};
