#pragma once

#include "TM.h"
#include <vector>
#include "orbBlock.h"
#include "StateVector.h"
#include "json_functions.h"

typedef std::vector<OrbBlock> OrbBlocks;

class OrbBlocksStore
{
public:
    OrbBlocksStore();
    void init(const SInitOrbits& orbits, const TimeJD* beg, const TimeJD* end, bool filter_orbits, bool min_dist_forced, bool interpolate);
    void init(const SInitOrbits& orbits, const TimeJD& beg, const TimeJD& end, bool filter_orbits, bool min_dist_forced, bool interpolate);
    SInitOrbits calc_orbits(std::vector<TimeJD> dates, bool variates) const;
    Vect6 get_pos(const TimeJD& date) const;
    const SInitOrbit& get_orbit(const TimeJD& date) const;
    const OrbBlocks& get_blocks() const { return m_blocks; }
    OrbBlocks& get_blocks() { return m_blocks; }
    const TimeJD& get_beg_before_filter() const { return m_beg_before_filter; }
    const TimeJD& get_end_before_filter() const { return m_end_before_filter; }
    void load(const jsonval& parent);
private:
    void init_min_dist(const SInitOrbits& orbits, const TimeJD& beg, const TimeJD& end, bool forced);
    void calc_min_dist(const SInitOrbit& orb1, const SInitOrbit& orb2, const TimeJD& beg, const TimeJD& end, TimeJD& result_jd, double& result_dist) const;
    SInitOrbits filter(const SInitOrbits& orbits) const;
    OrbBlocks m_blocks;
    TimeJD m_beg_before_filter;
    TimeJD m_end_before_filter;
};
