#pragma once

#include <algorithm>
#include <cmath>

template <typename T>
double golden_section_search(const T& f, double a, double b, double tol)
{
    double invphi = (std::sqrt(5.0) - 1) / 2.0;
    double invphi2 = (3 - std::sqrt(5.0)) / 2.0;

    a = std::min(a, b);
    b = std::max(a, b);

    double h = b - a;

    if (h <= tol) {
        return (a + b) / 2.0;
    }

    int n = (int)ceil(std::log(tol / h) / std::log(invphi));

    double c = a + invphi2 * h;
    double d = a + invphi * h;
    double yc = f.calc(c);
    double yd = f.calc(d);

    for (int k = 0; k < n - 1; k++) {
        if (yc < yd) {
            b = d;
            d = c;
            yd = yc;
            h = invphi * h;
            c = a + invphi2 * h;
            yc = f.calc(c);
        }
        else {
            a = c;
            c = d;
            yc = yd;
            h = invphi * h;
            d = a + invphi * h;
            yd = f.calc(d);
        }
    }

    if (yc < yd) {
        return (a + d) / 2.0;
    }
    else {
        return (c + b) / 2.0;
    }
}
